function eknn = getEnclosingKNN_qp(X, Z, limK, custeps)
  if nargin < 4,
    custeps = 1e-4;
  end  

  [nX dX] = size(X);
  [nZ dZ] = size(Z);
  if dX~=dZ,
    error('Dimension mismatch!');
  end

  %If unspecified, set maximal limits on K
  if nargin<3
    limK = [1 nX];
  elseif isinf(limK(2))
    limK(2) = nX;
  end   

  %Initialize matrices to increase performance
  eknn.finalNC  = zeros(1,nZ);
  eknn.indices  = zeros(nZ,limK(2));

  for i = 1:nZ
      Xc = X - repmat(Z(i,:),[nX,1]);
      [Dsort Isort] = sort(sum(Xc.^2, 2), 'ascend');
      minDist = sqrt(Dsort(1));
      Xsort = Xc(Isort,:);
      k = 1;   
      
      %Find the hyperplane normal 
      unitvector  = repmat(Xsort(1,:),[nX,1]) / minDist;
      projections = sum( Xsort.*unitvector, 2 );

      %Loop until out of points of in the convex hull
      while k<limK(2)
        %view2d(Xsort,1:k);
        
        %Find the nearest point that can bring the convex hull closer
        newk = find(projections<minDist & (1:nX)'>k, 1, 'first');      
        if isempty(newk)
          break;
        else
          k = newk;
        end    
        
        %Find the distance to enclosure for this neighborhood
        [minDist,weights,xhat] = minDist2Hull(zeros(1,dX),Xsort(1:k,:));

        %Break if you have found the convex Hull
        if (minDist<custeps), 
          minDist = 0;
          break; 
        end

        %Find all points on the near side of the hyperplane
        unitvector = repmat(xhat,[nX,1])/minDist;
        projections = sum( Xsort.*unitvector,2 );
      end

      %Bound the K to the desired range
      if k>limK(2), k = limK(2); end    
      if k<limK(1), k = limK(1); end

      eknn.finalNC(i) = minDist;
      eknn.indices(i,1:k) = Isort(1:k);         
  end

end

function view2d(X,I)
  scatter2d =@(X,c) scatter(X(:,1),X(:,2),c,'filled');
  figure(1); clf; hold on; axis equal;
  scatter2d([0 0],'g')
  scatter2d(X,'b');
  scatter2d(X(I,:),'r');
end
