%Projects testPt to the convex hull of trainPts and computes the minimum
%distance to this convex hull.
%Input:   testPt = 1*d vector
%         trainPts = N*d matrix
%Output:  minNC = scalar
%version 2.0
function [nc,weights,proj] = minDist2Hull(testPt,trainPts)
 
  numTest = size(testPt,1);
  numTrain = size(trainPts,1);
  
  %Sanity Check
  if numTest > 1
    error('Incorrect dimensions for testPt');
  end
  
  %If the testPt is the origin (the trainPts have been centered) then no
  %need to recenter
  if sum( testPt(:)~=0 ) == 0 %the only way the sum will not resolve to zero is if testPt is not the origin
    hPts = trainPts;
  else
    hPts = trainPts-repmat(testPt,[numTrain,1]);
  end
  
  H = hPts*hPts';

  %Quad Programming options
  f = zeros(numTrain,1);
  lb = zeros(1,numTrain);  
  OneConstraintA = ones(1,numTrain);
  OneConstraintb = 1;
  lpopt = optimset('Display','off', 'LargeScale', 'off');

  % x = quadprog(H,f,A,b,Aeq,beq,lb,ub,x0,options)
  %[weights, soln, exitflag] = quadprog(H,f,[],[],OneConstraintA,OneConstraintb,lb,[],[],lpopt);
  [weights, soln, exitflag] = quadprog(H,f,[],[],OneConstraintA,OneConstraintb,lb,[],[],lpopt);

  proj = weights'*trainPts;
  nc   = sqrt(sum((testPt-proj).^2));  
    
  
