function K = graphHessian(nGrid)

  % nGrid is a vector with the number of nodes in each dimension of the grid
  dX = length(nGrid); % number of dimensions of the grid
  nX = prod(nGrid);   % total number of nodes in the grid

  % the kernel primitives
  H = [1 -2 1; -2 4 -2; 1 -2 1];
  pI = double(dec2base(0:9-1, 3))-49;
  for d=1:dX 
      pairs(d).a = [zeros(9,d-1) pI(:,1) zeros(9,dX-d)];
      pairs(d).b = [zeros(9,d-1) pI(:,2) zeros(9,dX-d)];
  end

  % For every row in the kernel, there are at most 3^dX non-zeros elements. 
  K = spalloc(nX,nX, 3^dX * nX);

  index = cell(1,dX);
  for i=1:nX
    % find the position of this index in the original dX grid
    [index{:}] = ind2sub(nGrid,i);
    pos = cell2mat(index);
    pos_rep = repmat(pos,[9,1]);

    % for each dimension add (y_{i-1} - 2y_{i} + y_{i+1})^2 for the 
    % adjacent nodes in that dimension
    for d=1:dX
      
      % if you are at the border in this dimension, skip
      if (pos(d)==1) || (pos(d)==nGrid(d))
        continue;
      end

      % find the pairs that will be updated 
      marks.a = mat2cell(pos_rep + pairs(d).a, 9, ones(dX,1));
      marks.b = mat2cell(pos_rep + pairs(d).b, 9, ones(dX,1));    

      % find the index of these pairs in the kernel
      K_a = sub2ind(nGrid,marks.a{:});
      K_b = sub2ind(nGrid,marks.b{:});

      % find the linear entry in the kernel for these pairs
      insert = sub2ind(size(K),K_a,K_b);
      
      % update these pairs in the kernel
      K(insert) = K(insert) + H(:);
    end
  end
end