function test_2D_Regression(loadflag)
  addpath latticeRegression
  addpath eknn
  addpath gpml
  
  if exist('loadflag','var') && loadflag~=0
    load('f.mat');
  else
    f = gmm(2*(rand(2,10)-.25),.1);
    save('f.mat','f');
  end

  %Create a grid of the desired size for your input variables
  gr.n = [33,33];
  gr.x = getGrid(gr.n,[0,1;0,1]);
  gr.y = f(gr.x);
  gr.name = 'True Function';

  %Scaling function adds a buffer for the lattice
  rescale = @(x) x .* repmat(1-2./(gr.n'-1),[1,size(x,2)]) +...
                      repmat(1./(gr.n'-1),[1,size(x,2)]);  
  
  %Randomly sample some points from the function
  n = 100;
  train.x = rescale(rand(2,n));
  train.y = f(train.x);

  %Get some random test data
  test.x = rescale(rand(2,1000));
  test.y = f(test.x);

  %Create a plotter function
  set(figure(10),'name','True'); 
  plotLUT(gr,train);

%% Start  
  i=0;
  result = evaluateLUT(); %initialize

%%  Lattice Regression
  i=i+1; g(i) = gr;
  g(i).name = 'Lattice Regression';
  timer.start = clock;
  g(i).y = latticeRegression(train.x',train.y',g(i).x',1e-6);
  timer.etime = etime(clock,timer.start);
  
  result = evaluateLUT(result,g(i),train,test,timer.etime,figure(i));

%% Local Linear Regression eKNN (tikhonov regression)
  i=i+1; g(i) = gr;
  g(i).name = 'Tikhonov';  
  timer.start = clock;
  regress = @(X1,Y1,X2) regress_LocalLinear(X1,Y1,X2,'eknn',[5,30],'tikhonov',.1);
  g(i).y = regress(train.x',train.y',g(i).x')';
  timer.etime = etime(clock,timer.start);

  result = evaluateLUT(result,g(i),train,test,timer.etime,figure(i));  
  
%% Gaussian Process Regression
  i = i+1; g(i) = gr;
  g(i).name = 'Gaussian Process Regression';
  timer.start = clock;
  covfunc = {'covSum', {'covSEiso','covNoise'}};  
  loghyper = minimize([-1; -1; -1], 'gpr', -100, covfunc, train.x', train.y');
  [y S2] = gpr(loghyper, covfunc, train.x', train.y', g(i).x');
  g(i).y = y';
  timer.etime = etime(clock,timer.start);
  
  result = evaluateLUT(result,g(i),train,test,timer.etime,figure(i)); 

%% Print Results
  fprintf('%50s   %10s %10s\n','Name','Tr RMSE','Te RMSE');  
  for j=1:length(result.name)
    fprintf('%50s   %10.5f %10.5f\n',result.name{j},...
      sqrt(result.trainerr(j)),sqrt(result.testerr(j)));    
  end  
  
end

function result = evaluateLUT(result,g,train,test,etime,figh)
  %Initialization
  if ~exist('result','var')
    result = struct('name',[],'etime',[],'trainerr',[],'testerr',[]);
    return;
  end

  result.name{end+1} = g.name;
  result.etime(end+1) = etime;
  
  y = applyLUT(train.x,g.x,g.y);
  result.trainerr(end+1) = mean((y-train.y).^2);

  y = applyLUT(test.x,g.x,g.y);
  result.testerr(end+1) = mean((y-test.y).^2);
  
  set(figh,'name',g.name); 
  plotLUT(g,train);    
end


