function test_3D_Regression(loadflag)

if exist('loadflag','var') && loadflag~=0
  load('f.mat');
else
  f = gmm(2*(rand(3,10)-.25),.1);
  save('f.mat','f');
end

%Create a grid
gr.n = [17,17,17];
gr.x = getGrid(gr.n,[0,1;0,1;0,1]);
gr.y = f(gr.x);
gr.name = 'True Function';

%Scaling function adds a buffer for the lattice
rescale = @(x) x .* repmat(1-2./(gr.n'-1),[1,size(x,2)]) +...
                    repmat(1./(gr.n'-1),[1,size(x,2)]);

%Randomly sample some points from the function
n = 100;
train.x = rescale(rand(3,n));
train.y = f(train.x);

%Get some random test data
test.x = rescale(rand(3,1000));
test.y = f(test.x);    

%Set up the plotting function, tune the desired number of steps.
nSteps = 50;
lim = [min(gr.y(:)) max(gr.y(:))];
values = (0:1/(nSteps+1):1)*(lim(2)-lim(1)) + lim(1);

%Plot the true function
set(figure(10),'name','True Function'); 
plotLUT(gr,values,train);


%% Try out the regression techniques
i=0;
result = evaluateLUT(); %initialize

%% Lattice Regression
i=i+1; g(i) = gr;
g(i).name = 'Lattice Regression';
timer.start = clock;
g(i).y = latticeRegression(train.x',train.y',g(i).x',1e-6);
timer.etime = etime(clock,timer.start);

result = evaluateLUT(result,g(i),train,test,timer.etime);
set(figure(i),'name',g(i).name); 
plotLUT(g(i),values,train);

%% Local Linear Regression eKNN (tikhonov regression)
%% (WARNING: Reeeeeally Slow!!)
% i=i+1; g(i) = gr;
% g(i).name = 'Tikhonov';
% timer.start = clock;
% g(i).y = regress_LocalLinear(train.x',train.y',g(i).x','eknn',[3,Inf],'tikhonov',1)';
% timer.etime = etime(clock,timer.start);
% 
% result = evaluateLUT(result,g(i),train,test,timer.etime);
% set(figure(i),'name',g(i).name); 
% plotLUT(g(i),values,train);

%% Gaussian Process Regression
i = i+1; g(i) = gr;
g(i).name = 'Gaussian Process Regression';
timer.start = clock;
covfunc = {'covSum', {'covSEiso','covNoise'}};
loghyper = minimize([-1; -1; -1], 'gpr', -100, covfunc, train.x', train.y');
[y S2] = gpr(loghyper, covfunc, train.x', train.y', g(i).x');
g(i).y = y';
timer.etime = etime(clock,timer.start);

result = evaluateLUT(result,g(i),train,test,timer.etime);
set(figure(i),'name',g(i).name); 
plotLUT(g(i),values,train); 

%% Print the Result
  fprintf('%50s   %10s %10s\n','Name','Tr RMSE','Te RMSE');  
  for j=1:length(result.name)
    fprintf('%50s   %10.5f %10.5f\n',result.name{j},...
      sqrt(result.trainerr(j)),sqrt(result.testerr(j)));    
  end  

end

function result = evaluateLUT(result,g,train,test,etime)
  %Initialization
  if ~exist('result','var')
    result = struct('name',[],'etime',[],'trainerr',[],'testerr',[]);
    return;
  end

  result.name{end+1} = g.name;
  result.etime(end+1) = etime;
  
  y = applyLUT(train.x,g.x,g.y);
  result.trainerr(end+1) = mean((y-train.y).^2);

  y = applyLUT(test.x,g.x,g.y);
  result.testerr(end+1) = mean((y-test.y).^2); 
   
end


